#!/usr/bin/python
# -*- coding:utf-8 -*-
import time
import random
from .bread_handle import BreadDataManage, Action, BreadData
from .config import MAX, MIN, CD, LEVEL, bread_config
from enum import Enum


def cd_wait_time(group_id, user_id, operate: Action) -> int:
    """获取需要等待的CD秒数，小于0则被ban，大于0则还在冷却，等于0则可操作"""
    cd_stamp = BreadDataManage(group_id).cd_get_stamp(user_id, operate)
    now_stamp = int(time.time())
    sep_time = now_stamp - cd_stamp
    cd = getattr(CD, operate.name)
    if sep_time < 0:
        return sep_time
    return cd.value - sep_time if sep_time < cd.value else 0


class _Event:
    event_type = Action.ALL
    _instance = {}
    _has_init = {}
    _public_events = []
    _is_random = {}
    _is_random_global = True

    def __new__(cls, group_id: str):
        """事件关于群单例，每个群只能有一个事件实例"""
        if group_id is None:
            return None
        if cls._instance.get(group_id) is None:
            cls._instance[group_id] = super(_Event, cls).__new__(cls)
        return cls._instance[group_id]

    def __init__(self, group_id: str):
        if not self._has_init.get(group_id):
            self._has_init[group_id] = True
            self.group_id = group_id
            self.bread_db = BreadDataManage(group_id)
            self.user_id = None
            self.user_data = BreadData(0, "0", 0, 0, 0)
            self._private_events = []
            self.thing = bread_config.special_thing_group.get(group_id, bread_config.bread_thing)
            if isinstance(self.thing, list):
                self.thing = self.thing[0]

    @classmethod
    def add_event(cls, func):
        """添加单个特殊事件"""
        if not func.group_id_list:
            cls._public_events.append(func)
        else:
            for group_id in func.group_id_list:
                cls(group_id)._private_events.append(func)

    @classmethod
    def add_events(cls, func_list):
        """添加多个特殊事件"""
        for func in func_list:
            cls.add_event(func)

    @classmethod
    def set_random_global(cls, flag):
        """全局是否随机"""
        cls._is_random_global = flag

    def set_random(self, flag):
        """设置是否随机操作数量，若为False，可以通过指令指定数量"""
        self._is_random[self.group_id] = flag

    def is_random(self):
        """获取 self.group_id 群组是否随机"""
        if self._is_random.get(self.group_id) is not None:
            return self._is_random.get(self.group_id)
        else:
            return self._is_random_global

    def _special_event(self):
        """按照优先级排布，同优先级随机排布"""
        events = self._private_events + self._public_events
        events.sort(key=lambda x: (x.priority, random.random()))
        for event_func in events:
            return_data = event_func(self)
            if return_data:
                return return_data

    def set_user_id(self, user_id: str):
        """设置用户id以及获取用户数据以待判断"""
        self.user_id = user_id
        self.user_data = self.bread_db.get_bread_data(self.user_id)

    def execute(self, num=None):
        """事件执行"""
        pre_error = self._pre_event(num)
        if pre_error:
            return pre_error

        return_data = self._special_event()
        self.bread_db.add_user_log(self.user_id, self.event_type)
        if return_data:
            return return_data

        return self.normal_event()

    def normal_event(self):
        """正常事件流程 （虚函数）"""

    def _pre_event(self, num=None):
        """预处理事件，提前生成随机值或其它值。 num 为非随机情况下得指定数量值"""
        self.thing = bread_config.special_thing_group.get(self.group_id, bread_config.bread_thing)
        if isinstance(self.thing, list):
            self.thing = self.thing[0]

        # 获取操作最大值或者最小值
        max_num = getattr(MAX, self.event_type.name).value
        min_num = getattr(MIN, self.event_type.name).value

        if not self.is_random() and num is not None:
            if min_num <= num <= max_num:
                self.action_num = num
                return
            else:
                return "数量和限制不符！"
        # self.action_num = random.randint(min_num, min(max_num, self.user_data.bread_num))


class _Event2(_Event):
    """双用户事件"""

    def __init__(self, group_id: str):
        super().__init__(group_id)
        self.other_id = None
        self.other_name = None
        self.other_data = BreadData(0, "0", 0, 0, 0)

    def set_other_id(self, other_id: str, other_name: str):
        """设置第二个用户id 通常为”被操作“的用户"""
        self.other_id = other_id
        self.other_name = other_name
        self.other_data = self.bread_db.get_bread_data(other_id)


class BuyEvent(_Event):
    """
    买事件，用户增加面包
    """
    event_type = Action.BUY
    _instance = {}
    _has_init = {}
    _public_events = []
    _is_random = {}
    _is_random_global = True

    def normal_event(self):
        new_bread_num = self.bread_db.add_bread(self.user_id, self.action_num)
        new_bread_no = self.bread_db.update_no(self.user_id)
        append_text = f"成功购买了{self.action_num}个{self.thing}w，现在一共拥有{new_bread_num}个{self.thing}！您的{self.thing}排名为:{new_bread_no}"
        self.bread_db.cd_update_stamp(self.user_id, Action.BUY)
        return append_text

    def _pre_event(self, num=None):
        self.action_num = random.randint(MIN.BUY.value, MAX.BUY.value)
        return super(BuyEvent, self)._pre_event(num)


class EatEvent(_Event):
    """
    吃事件，用户减少面包，增加“已吃面包”数量，在一定值等级会随之提高
    """
    event_type = Action.EAT
    _instance = {}
    _has_init = {}
    _public_events = []
    _is_random = {}
    _is_random_global = True

    def normal_event(self):
        now_bread = self.bread_db.reduce_bread(self.user_id, self.action_num)
        eaten_bread = self.bread_db.add_bread(self.user_id, self.action_num, Action.EAT)
        append_text = f"成功吃掉了{self.action_num}个{self.thing}w！现在你还剩{now_bread}个{self.thing}w！您目前的等级为Lv.{eaten_bread // LEVEL}"
        self.bread_db.cd_update_stamp(self.user_id, Action.EAT)
        self.bread_db.update_no(self.user_id)
        return append_text

    def _pre_event(self, num=None):
        if self.user_data.bread_num < MIN.EAT.value or (num and self.user_data.bread_num < num):
            append_text = f"你的{self.thing}还不够吃w，来买一些{self.thing}吧！"
            return append_text

        self.action_num = random.randint(MIN.EAT.value, min(MAX.EAT.value, self.user_data.bread_num))
        return super(EatEvent, self)._pre_event(num)


class BetEvent(_Event):
    """
    猜拳事件，由用户选择“石头”“剪刀”“石头”中的一个，bot将会随机生成一个手势
    若用户胜利，用户获得面包，若失败，则丢失面包。
    """
    event_type = Action.BET
    _instance = {}
    _has_init = {}
    _public_events = []
    _is_random = {}
    _is_random_global = True

    class G(Enum):
        ROCK = 0
        PAPER = 1
        SCISSORS = 2

    class RES(Enum):
        DRAW = 0  # 平局
        WIN = 1
        LOST = 2

    def __init__(self, group_id: str):
        super().__init__(group_id)
        self.user_gestures = None  # 额外初始化一个手势变量

    def set_user_gestures(self, ges: G):
        """猜拳，设置用户的手势"""
        self.user_gestures = ges

    def normal_event(self):
        if self.outcome == self.RES.WIN:
            new_bread_num_user = self.bread_db.add_bread(self.user_id, self.action_num)
            self.bread_db.update_no(self.user_id)
            append_text = f"{self.bot_ges_text}! 呜呜，我输了，给你{self.action_num}个{self.thing}！你现在拥有{new_bread_num_user}个{self.thing}！"
            self.bread_db.cd_update_stamp(self.user_id, Action.BET)
        elif self.outcome == self.RES.DRAW:
            append_text = f"{self.bot_ges_text}!平局啦！{self.thing}都还给你啦！还可以再来一次w！"
        else:  # self.outcome == self.RES.LOST
            new_bread_num_user = self.bread_db.reduce_bread(self.user_id, self.action_num)
            self.bread_db.update_no(self.user_id)
            append_text = f"{self.bot_ges_text}!嘿嘿，我赢啦！你的{self.action_num}个{self.thing}归我了！你现在拥有{new_bread_num_user}个{self.thing}！"
            self.bread_db.cd_update_stamp(self.user_id, Action.BET)
        return append_text

    def _pre_event(self, num=None):
        if self.user_data.bread_num < MIN.BET.value or (num and self.user_data.bread_num < num):
            append_text = f"你的{self.thing}还不够猜拳w，来买一些{self.thing}吧！"
            return append_text

        self.bot_ges = self.G(random.randint(0, 2))
        self.bot_ges_text = ("石头", "布", "剪刀")[self.bot_ges.value]
        val = self.user_gestures.value - self.bot_ges.value
        if val == -2 or val == -1:
            val = 3 + val
        self.outcome = self.RES(val)

        self.action_num = random.randint(MIN.BET.value, min(MAX.BET.value, self.user_data.bread_num))
        return super(BetEvent, self)._pre_event(num)


class RobEvent(_Event2):
    """
    抢劫事件，用户1获得面包，用户2减少面包
    """
    event_type = Action.ROB
    _instance = {}
    _has_init = {}
    _public_events = []
    _is_random = {}
    _is_random_global = True

    def normal_event(self):
        new_bread_num = self.bread_db.add_bread(self.user_id, self.action_num)
        self.bread_db.reduce_bread(self.other_id, self.action_num)
        new_bread_no = self.bread_db.update_no(self.user_id)
        self.bread_db.update_no(self.other_id)

        append_text = f"成功抢了{self.other_name}{self.action_num}个{self.thing}，你现在拥有{new_bread_num}个{self.thing}！" \
                      f"您的{self.thing}排名为:{new_bread_no}"
        self.bread_db.cd_update_stamp(self.user_id, Action.ROB)
        return append_text

    def _pre_event(self, num=None):
        if not self.other_data or self.other_data.bread_num < MIN.ROB.value\
                or (num and self.other_data.bread_num < num):
            append_text = f"{self.other_name}没有那么多{self.thing}可抢呜"
            return append_text

        self.action_num = random.randint(MIN.ROB.value, min(MAX.ROB.value, self.other_data.bread_num))
        pre_res = super(RobEvent, self)._pre_event(num)
        if pre_res:  # 有返回值代表事件提前结束
            return pre_res

        if self.user_id == self.other_id:
            reduce_num = min(self.action_num, self.user_data.bread_num)
            append_text = f"这么想抢自己哇，那我帮你抢！抢了你{reduce_num}个{self.thing}嘿嘿！"
            self.bread_db.reduce_bread(self.user_id, reduce_num)
            self.bread_db.update_no(self.other_id)
            self.bread_db.cd_update_stamp(self.user_id, Action.ROB)
            return append_text


class GiveEvent(_Event2):
    """
    赠送事件，用户1失去面包，用户2获得面包
    """
    event_type = Action.GIVE
    _instance = {}
    _has_init = {}
    _public_events = []
    _is_random = {}
    _is_random_global = True

    def normal_event(self):
        new_bread_num_given = self.bread_db.add_bread(self.other_id, self.action_num)
        new_bread_num_user = self.bread_db.reduce_bread(self.user_id, self.action_num)
        self.bread_db.update_no(self.other_id)
        self.bread_db.update_no(self.user_id)

        append_text = f"成功赠送了{self.action_num}个{self.thing}给{self.other_name}，你现在拥有{new_bread_num_user}个{self.thing}！" \
                      f"{self.other_name}有{new_bread_num_given}个{self.thing}！"
        self.bread_db.cd_update_stamp(self.user_id, Action.GIVE)
        return append_text

    def _pre_event(self, num=None):
        if self.user_data.bread_num < MIN.GIVE.value or (num and self.user_data.bread_num < num):
            append_text = f"你的{self.thing}还不够赠送w，来买一些{self.thing}吧！"
            return append_text

        self.action_num = random.randint(MIN.GIVE.value, min(MAX.GIVE.value, self.user_data.bread_num))
        pre_res = super(GiveEvent, self)._pre_event(num)
        if pre_res:  # 有返回值代表事件提前结束
            return pre_res

        if self.user_id == self.other_id:
            append_text = f"送给自己肯定是不行的啦！送给我叭！你送了我{self.action_num}个{self.thing}嘿嘿！"
            self.bread_db.reduce_bread(self.user_id, self.action_num)
            self.bread_db.update_no(self.user_id)
            self.bread_db.cd_update_stamp(self.user_id, Action.GIVE)
            return append_text



#!/usr/bin/python
# -*- coding:utf-8 -*-
import sqlite3
import time

from collections import namedtuple
from enum import Enum
from functools import wraps
from inspect import signature
from pathlib import Path
from typing import List

from .config import LEVEL

DATABASE = Path() / "data" / "bread"


class Action(Enum):
    ALL = -1
    BUY = 0
    EAT = 1
    ROB = 2
    GIVE = 3
    BET = 4


BreadData = namedtuple("BreadData", ["no", "user_id", "bread_num", "bread_eaten", "level"])
LogData = namedtuple("LogData", ["user_id", "buy_times", "eat_times", "rob_times", "give_times", "bet_times"])


def type_assert(*ty_args, **ty_kwargs):
    """sql类型检查"""
    def decorate(func):
        sig = signature(func)
        bound_types = sig.bind_partial(*ty_args, **ty_kwargs).arguments

        @wraps(func)
        def wrapper(*args, **kwargs):
            bound_values = sig.bind(*args, **kwargs)
            for name, value in bound_values.arguments.items():
                if name in bound_types:
                    if bound_types[name] == "user_id":
                        if isinstance(value, str):
                            if not value.isdigit():
                                raise TypeError("user_id must consist of numeric characters.")
                        else:
                            raise TypeError('Argument {} must be {}'.format(name, str))
                    elif not isinstance(value, bound_types[name]):
                        raise TypeError('Argument {} must be {}'.format(name, bound_types[name]))
            return func(*args, **kwargs)
        return wrapper
    return decorate


class BreadDataManage:
    _instance = {}
    _has_init = {}
    CD_COLUMN = ("BUY_CD", "EAT_CD", "ROB_CD", "GIVE_CD", "BET_CD")
    DATA_COLUMN = ("BREAD_NUM", "BREAD_EATEN")
    LOG_COLUMN = ("BUY_TIMES", "EAT_TIMES", "ROB_TIMES", "GIVE_TIMES", "BET_TIMES")

    def __new__(cls, group_id):
        if group_id is None:
            return None
        if cls._instance.get(group_id) is None:
            cls._instance[group_id] = super(BreadDataManage, cls).__new__(cls)
        return cls._instance[group_id]

    def __init__(self, group_id):
        if not BreadDataManage._has_init.get(group_id):
            BreadDataManage._has_init[group_id] = True
            self.database_path = DATABASE / group_id
            if not self.database_path.exists():
                self.database_path.mkdir(parents=True)
                self.database_path /= "bread.db"
                self.conn = sqlite3.connect(self.database_path)
                self._create_file()
            else:
                self.database_path /= "bread.db"
                self.conn = sqlite3.connect(self.database_path)
            print(f"群组{group_id}数据库连接！")

    def close(self):
        self.conn.close()
        print("数据库关闭！")

    def _create_file(self) -> None:
        """创建数据库文件"""
        c = self.conn.cursor()
        c.execute('''CREATE TABLE BREAD_DATA
                           (NO            INTEGER PRIMARY KEY UNIQUE,
                           USERID         TEXT     ,
                           BREAD_NUM      INTEGER  ,
                           BREAD_EATEN    INTEGER  
                           );''')
        c.execute('''CREATE TABLE BREAD_LOG
                           (USERID         TEXT     ,
                           BUY_TIMES      INTEGER  ,
                           EAT_TIMES      INTEGER  ,
                           ROB_TIMES      INTEGER  ,
                           GIVE_TIMES     INTEGER  ,
                           BET_TIMES      INTEGER  
                           );''')
        c.execute('''CREATE TABLE BREAD_CD
                           (USERID        TEXT     ,
                           BUY_CD         INTEGER  ,
                           EAT_CD         INTEGER  ,
                           ROB_CD         INTEGER  ,
                           GIVE_CD        INTEGER  ,
                           BET_CD         INTEGER  
                           );''')
        self.conn.commit()

    def _get_id(self) -> int:
        """获取下一个id"""
        cur = self.conn.cursor()
        cur.execute('select * from BREAD_DATA')
        result = cur.fetchall()
        return len(result) + 1

    @classmethod
    def close_dbs(cls):
        for group_id in cls._instance.keys():
            BreadDataManage(group_id).close()

    @type_assert(object, "user_id")
    def _create_user(self, user_id: str) -> None:
        """在数据库中创建用户并初始化"""
        new_id = self._get_id()
        c = self.conn.cursor()
        sql = f"INSERT INTO BREAD_DATA (NO,USERID,BREAD_NUM,BREAD_EATEN) VALUES (?,?,0,0)"
        c.execute(sql, (new_id, user_id))
        sql = f"INSERT INTO BREAD_LOG (USERID,BUY_TIMES,EAT_TIMES,ROB_TIMES,GIVE_TIMES,BET_TIMES)" \
              f" VALUES (?,0,0,0,0,0)"
        c.execute(sql, (user_id,))
        sql = f"INSERT INTO BREAD_CD (USERID,BUY_CD,EAT_CD,ROB_CD,GIVE_CD,BET_CD) VALUES (?,0,0,0,0,0)"
        c.execute(sql, (user_id,))
        self.conn.commit()

    @type_assert(object, "user_id", Action)
    def cd_refresh(self, user_id: str, action: Action) -> None:
        """刷新用户操作cd"""
        if action == Action.ALL:
            cur = self.conn.cursor()
            for key in self.CD_COLUMN:
                sql = f"update BREAD_CD set {key}=? where USERID=?"
                cur.execute(sql, (1, user_id))
            self.conn.commit()
            return
        op_key = self.CD_COLUMN[action.value]
        sql = f"update BREAD_CD set {op_key}=? where USERID=?"
        cur = self.conn.cursor()
        cur.execute(sql, (1, user_id))
        self.conn.commit()

    @type_assert(object, "user_id", Action)
    def cd_get_stamp(self, user_id: str, action: Action) -> int:
        """获取用户上次操作时间戳"""
        cur = self.conn.cursor()
        sql = f"select * from BREAD_CD where USERID=?"
        cur.execute(sql, (user_id,))
        result = cur.fetchone()
        if not result:
            self._create_user(user_id)
            result = (user_id, 0, 0, 0, 0, 0)
        self.conn.commit()
        return result[action.value + 1]

    @type_assert(object, "user_id", Action, int)
    def cd_reduce_action(self, user_id: str, action: Action, reduce_time) -> None:
        """单次剪短cd，单位为秒"""
        op_key = self.CD_COLUMN[action.value]
        sql = f"update BREAD_CD set {op_key}=? where USERID=?"
        cur = self.conn.cursor()
        cur.execute(sql, (int(time.time()) - reduce_time, user_id))
        self.conn.commit()

    @type_assert(object, "user_id", Action, int)
    def cd_ban_action(self, user_id: str, action: Action, ban_time) -> None:
        """禁止用户一段时间操作，单次延长cd，单位为秒"""
        op_key = self.CD_COLUMN[action.value]
        sql = f"update BREAD_CD set {op_key}=? where USERID=?"
        cur = self.conn.cursor()
        cur.execute(sql, (int(time.time()) + ban_time, user_id))
        self.conn.commit()

    @type_assert(object, "user_id", Action)
    def cd_update_stamp(self, user_id: str, action: Action) -> None:
        """重置用户操作CD(重新开始记录冷却)"""
        op_key = self.CD_COLUMN[action.value]
        sql = f"update BREAD_CD set {op_key}=? where USERID=?"
        stamp = int(time.time())
        cur = self.conn.cursor()
        cur.execute(sql, (stamp, user_id))
        self.conn.commit()

    @type_assert(object, "user_id", int, Action)
    def add_bread(self, user_id: str, add_num: int, action: Action = Action.BUY) -> int:
        """添加用户面包数量，可以添加已经吃了的面包数量，返回增加后的数量"""
        if action == Action.EAT:
            col_name = self.DATA_COLUMN[1]
        else:
            col_name = self.DATA_COLUMN[0]
        cur = self.conn.cursor()
        sql = f"select * from BREAD_DATA where USERID=?"
        cur.execute(sql, (user_id,))
        data = cur.fetchone()
        if not data:
            self._create_user(user_id)
            data = (0, user_id, 0, 0)
        if col_name == self.DATA_COLUMN[1]:
            ori_num = data[3]
        else:
            ori_num = data[2]
        new_num = ori_num + add_num
        sql = f"update BREAD_DATA set {col_name}=? where USERID=?"
        cur.execute(sql, (new_num, user_id))
        self.conn.commit()
        return new_num

    @type_assert(object, "user_id", int, Action)
    def reduce_bread(self, user_id: str, red_num: int, action: Action = Action.BUY) -> int:
        """减少用户面包数量，可以减少已经吃的数量，返回减少后的数量"""
        if action == Action.EAT:
            col_name = self.DATA_COLUMN[1]
        else:
            col_name = self.DATA_COLUMN[0]
        cur = self.conn.cursor()
        sql = "select * from BREAD_DATA where USERID=?"
        cur.execute(sql, (user_id,))
        data = cur.fetchone()
        if not data:
            self._create_user(user_id)
            data = (0, user_id, 0, 0)
        if col_name == "BREAD_EATEN":
            ori_num = data[3]
        else:
            ori_num = data[2]
        new_num = ori_num - red_num
        sql = f"update BREAD_DATA set {col_name}=? where USERID=?"
        cur.execute(sql, (new_num, user_id))
        self.conn.commit()
        return new_num

    @type_assert(object, "user_id")
    def update_no(self, user_id: str) -> int:
        """更新用户排名并返回"""
        cur = self.conn.cursor()
        sql = "select * from BREAD_DATA where USERID=?"
        cur.execute(sql, (user_id,))
        data = cur.fetchone()
        now_no = data[0]
        user_num = (data[3] // LEVEL, data[2])
        while now_no != 1:
            cur.execute("select * from BREAD_DATA where NO=?", (now_no - 1,))
            data = cur.fetchone()
            up_num = (data[3] // LEVEL, data[2])
            if user_num > up_num:
                cur.execute(f"update BREAD_DATA set NO={0} where NO={now_no}")
                cur.execute(f"update BREAD_DATA set NO={now_no} where NO={now_no - 1}")
                cur.execute(f"update BREAD_DATA set NO={now_no - 1} where NO={0}")
            else:
                break
            now_no -= 1
        while now_no != self._get_id() - 1:
            cur.execute("select * from BREAD_DATA where NO=?", (now_no + 1,))
            data = cur.fetchone()
            down_num = (data[3] // LEVEL, data[2])
            if user_num < down_num:
                cur.execute("update BREAD_DATA set NO=? where NO=?", (0, now_no))
                cur.execute("update BREAD_DATA set NO=? where NO=?", (now_no, now_no + 1))
                cur.execute("update BREAD_DATA set NO=? where NO=?", (now_no + 1, 0))
            else:
                break
            now_no += 1
        self.conn.commit()
        return now_no

    @type_assert(object, "user_id")
    def get_bread_data(self, user_id: str) -> BreadData:
        """获取用户面包数据并返回"""
        cur = self.conn.cursor()
        cur.execute("select * from BREAD_DATA where USERID=?", (user_id,))
        data = cur.fetchone()
        if not data:
            self._create_user(user_id)
            data = (0, user_id, 0, 0)
        self.conn.commit()
        return BreadData(*data, level=data[3] // LEVEL)

    def get_all_data(self) -> List[BreadData]:
        """获取一个数据库内的所有用户数据"""
        cur = self.conn.cursor()
        cur.execute(f"select * from BREAD_DATA")
        data = cur.fetchall()
        self.conn.commit()
        return [BreadData(*item, level=item[3] // LEVEL) for item in data]

    @type_assert(object, "user_id", Action)
    def add_user_log(self, user_id: str, action: Action) -> int:
        """记录用户操作次数递增1并返回"""
        op_key = self.LOG_COLUMN[action.value]
        cur = self.conn.cursor()
        cur.execute("select * from BREAD_LOG where USERID=?", (user_id,))
        data = cur.fetchone()
        log_times = data[action.value + 1]
        log_times += 1
        cur.execute(f"update BREAD_LOG set {op_key}=? where USERID=?", (log_times, user_id))
        self.conn.commit()
        return log_times

    @type_assert(object, "user_id", Action)
    def reduce_user_log(self, user_id: str, action: Action) -> int:
        """记录用户操作次数递减1并返回"""
        op_key = self.LOG_COLUMN[action.value]
        cur = self.conn.cursor()
        cur.execute("select * from BREAD_LOG where USERID=?", (user_id,))
        data = cur.fetchone()
        log_times = data[action.value + 1]
        log_times -= 1
        cur.execute(f"update BREAD_LOG set {op_key}=? where USERID=?", (log_times, user_id))
        self.conn.commit()
        return log_times

    @type_assert(object, "user_id")
    def get_log_data(self, user_id: str) -> LogData:
        """获取用户操作次数记录数据"""
        cur = self.conn.cursor()
        cur.execute(f"select * from BREAD_LOG where USERID=?", (user_id,))
        data = cur.fetchone()
        self.conn.commit()
        return LogData(*data)

    @type_assert(object, Action)
    def get_action_log(self, action: Action) -> LogData:
        """获取某个操作次数最多用户的数据"""
        col = self.LOG_COLUMN[action.value]
        cur = self.conn.cursor()
        sql = f"SELECT * FROM BREAD_LOG WHERE {col}= (SELECT MAX({col}) FROM BREAD_LOG) LIMIT 1;"
        cur.execute(sql)
        data = cur.fetchone()
        self.conn.commit()
        return LogData(*data)


if __name__ == "__main__":
    DATABASE = Path() / ".." / ".." / ".." / "data" / "bread"



#!/usr/bin/python
# -*- coding:utf-8 -*-
from enum import Enum
from nonebot import get_driver
from pydantic import BaseSettings, Extra


class Config(BaseSettings, extra=Extra.ignore):
    bread_thing: str = "面包"  # 自定义物品，改掉默认的”面包“ 此为全局
    special_thing_group: dict = {}  # 分群设置物品 示例{"群号": "炸鸡"} 可设置主次物品词 示例{"群号": ["炸鸡", "面包", "蛋糕"]}
    # 注：主次物品词bot将以主物品词进行回复，次物品词只用于触发指令，主关键词为列表第一个元素，次关键词可多个

    global_bread: bool = True  # 面包默认开关
    black_bread_groups: list = []  # 黑名单
    white_bread_groups: list = []  # 白名单
    level_bread_num: int = 10  # 设置升一级所需要的面包数量（数据库不保存等级！！等级会随之而变！）

    global_database: bool = False  # 数据库是否全局，若设置了group_database，以其为优先，全局数据库文件夹名为"global"
    group_database: dict = {}  # 合并一些群的数据库 分组id将作为文件夹名 例：{"分组id":["群号1", "群号2", "群号3"]}
    # 注意：此处的分组id将生效于 special_thing_group 的设置 示例{"分组id": "炸鸡"}，原来的设置将失效
    # 特殊事件同理设置的群聊id同理请改为组id

    """操作冷却（单位：秒）"""
    cd_buy: int = 4200
    cd_eat: int = 4800
    cd_rob: int = 5000
    cd_give: int = 3000
    cd_bet: int = 5400

    """操作随机值上限"""
    max_buy: int = 9
    max_eat: int = 8
    max_rob: int = 7
    max_give: int = 10
    max_bet: int = 10

    """操作随机值下限"""
    min_buy: int = 1
    min_eat: int = 2
    min_rob: int = 2
    min_give: int = 1
    min_bet: int = 5

    """设置是否操作值都由随机值决定"""
    is_random_buy: bool = True
    is_random_eat: bool = True
    is_random_rob: bool = True
    is_random_give: bool = True
    is_random_bet: bool = True

    is_random_robbed: bool = True  # 抢面包操作不指定群员可随机抢
    is_random_given: bool = True  # 送面包操作不指定群员可随机送

    special_buy_group: dict = {}  # 示例： {"群号": bool}
    special_eat_group: dict = {}
    special_rob_group: dict = {}
    special_give_group: dict = {}
    special_bet_group: dict = {}

    """设置是否启用有效@"""
    is_at_valid: bool = False


global_config = get_driver().config
bread_config = Config(**global_config.dict())  # 载入配置


LEVEL = bread_config.level_bread_num


class CD(Enum):
    """操作冷却（单位：秒）"""
    BUY = bread_config.cd_buy
    EAT = bread_config.cd_eat
    ROB = bread_config.cd_rob
    GIVE = bread_config.cd_give
    BET = bread_config.cd_bet


class MAX(Enum):
    """操作随机值上限"""
    BUY = bread_config.max_buy
    EAT = bread_config.max_eat
    ROB = bread_config.max_rob
    GIVE = bread_config.max_give
    BET = bread_config.max_bet


class MIN(Enum):
    """操作随机值下限"""
    BUY = bread_config.min_buy
    EAT = bread_config.min_eat
    ROB = bread_config.min_rob
    GIVE = bread_config.min_give
    BET = bread_config.min_bet


def random_config():
    """设置操作数量是否由用户指定或随机"""
    from .bread_operate import BuyEvent, EatEvent, RobEvent, GiveEvent, BetEvent
    events = [BuyEvent, EatEvent, RobEvent, GiveEvent, BetEvent]
    global_settings = [bread_config.is_random_buy, bread_config.is_random_eat, bread_config.is_random_rob,
                       bread_config.is_random_give, bread_config.is_random_bet]
    special_settings = [bread_config.special_buy_group, bread_config.special_eat_group, bread_config.special_rob_group,
                        bread_config.special_give_group, bread_config.special_bet_group]

    for event_, setting in zip(events, global_settings):
        if not setting:
            event_.set_random_global(False)

    for event_, setting in zip(events, special_settings):
        if setting:
            for group_id in setting.keys():
                event_(group_id).set_random(setting[group_id])



#!/usr/bin/python
# -*- coding:utf-8 -*-
import random
import re
from itertools import chain

from nonebot import get_driver
from nonebot import on_command
from nonebot.params import CommandArg, RawCommand
from nonebot.adapters.onebot.v11 import Bot, Event, Message
from nonebot.exception import ActionFailed

from .bread_handle import BreadDataManage, Action
from .bread_operate import *
from .bread_event import rob_events, buy_events, eat_events, give_events, bet_events
from .config import LEVEL, random_config, bread_config

driver = get_driver()

# 基础命令列表（不管是什么物品）
cmd_buy_ori = {"buy", "🍞"}
cmd_eat_ori = {"eat", "🍞🍞"}
cmd_rob_ori = {"rob", "🍞🍞🍞"}
cmd_give_ori = {"give", "送"}
cmd_bet_ori = {"bet"}

cmd_log_ori = {"logb"}
cmd_check_ori = {"check"}

cmd_top_ori = {"breadtop"}
cmd_help_ori = {"breadhelp", "helpb"}

# 基础命令列表（根据物品添加触发）
cmd_buy = cmd_buy_ori.copy()
cmd_eat = cmd_eat_ori.copy()
cmd_rob = cmd_rob_ori.copy()
cmd_give = cmd_give_ori.copy()
cmd_bet = cmd_bet_ori.copy()

cmd_log = cmd_log_ori.copy()
cmd_check = cmd_check_ori.copy()

cmd_top = cmd_top_ori.copy()
cmd_help = cmd_help_ori.copy()
# 进行添加，拓展触发指令
for things in chain(bread_config.special_thing_group.values(), (bread_config.bread_thing,)):
    if isinstance(things, str):
        things = [things]
    for thing_ in things:
        cmd_buy.add(f"买{thing_}")
        cmd_eat.add(f"吃{thing_}")
        cmd_eat.add(f"啃{thing_}")
        cmd_rob.add(f"抢{thing_}")
        cmd_give.add(f"送{thing_}")
        cmd_bet.add(f"{thing_}猜拳")
        cmd_bet.add(f"赌{thing_}")

        cmd_log.add(f"{thing_}记录")
        cmd_check.add(f"偷看{thing_}")
        cmd_check.add(f"查看{thing_}")

        cmd_top.add(f"{thing_}排行")
        cmd_top.add(f"{thing_}排名")
        cmd_help.add(f"{thing_}帮助")

bread_buy = on_command("bread_buy", aliases=cmd_buy, priority=5)
bread_eat = on_command("bread_eat", aliases=cmd_eat, priority=5)
bread_rob = on_command("bread_rob", aliases=cmd_rob, priority=5)
bread_give = on_command("bread_give", aliases=cmd_give, priority=5)
bread_bet = on_command("bread_bet", aliases=cmd_bet, priority=5)

bread_log = on_command("bread_log", aliases=cmd_log, priority=5)
bread_check = on_command("bread_check", aliases=cmd_check, priority=5)
bread_top = on_command("bread_top", aliases=cmd_top, priority=5)
bread_help = on_command("bread_help", aliases=cmd_help, priority=5)

# 初始化事件
EatEvent.add_events(eat_events)
BuyEvent.add_events(buy_events)
RobEvent.add_events(rob_events)
GiveEvent.add_events(give_events)
BetEvent.add_events(bet_events)

# 设置是否可以指定操作数
# 例： ”/give @用户 10“即是否可以使用此处的 10
random_config()


@bread_buy.handle()
async def _(event: Event, bot: Bot, args: Message = CommandArg(), cmd: Message = RawCommand()):
    try:
        user_qq, group_id, name, msg_at, thing = await pre_get_data(event, bot, cmd, cmd_buy_ori)
        buy_num = get_num_arg(args.extract_plain_text(), BuyEvent, group_id)
    except ArgsError as e:
        await bot.send(event=event, message=str(e))
        return
    except CommandError:
        return

    wait_time = cd_wait_time(group_id, user_qq, Action.BUY)
    # 可见cd_wait_time函数的注释
    if wait_time > 0:
        data = BreadDataManage(group_id).get_bread_data(user_qq)
        msg_txt = f"您还得等待{wait_time // 60}分钟才能买{thing}w，现在一共拥有{data.bread_num}个{thing}！您的{thing}排名为:{data.no}"
    elif wait_time < 0:
        msg_txt = f"你被禁止购买{thing}啦！{(abs(wait_time) + CD.BUY.value) // 60}分钟后才能购买！"
    else:
        event_ = BuyEvent(group_id)
        event_.set_user_id(user_qq)
        msg_txt = event_.execute(buy_num)

    res_msg = msg_at + Message(msg_txt)
    await bot.send(event=event, message=res_msg)


@bread_eat.handle()
async def _(event: Event, bot: Bot, args: Message = CommandArg(), cmd: Message = RawCommand()):
    try:
        user_qq, group_id, name, msg_at, thing = await pre_get_data(event, bot, cmd, cmd_eat_ori)
        eat_num = get_num_arg(args.extract_plain_text(), EatEvent, group_id)
    except ArgsError as e:
        await bot.send(event=event, message=str(e))
        return
    except CommandError:
        return

    wait_time = cd_wait_time(group_id, user_qq, Action.EAT)
    if wait_time > 0:
        data = BreadDataManage(group_id).get_bread_data(user_qq)
        msg_txt = f"您还得等待{wait_time // 60}分钟才能吃{thing}w，现在你的等级是Lv.{data.bread_eaten // LEVEL}！您的{thing}排名为:{data.no}"
    elif wait_time < 0:
        msg_txt = f"你被禁止吃{thing}啦！{(abs(wait_time) + CD.EAT.value) // 60}分钟后才能吃哦！"
    else:
        event_ = EatEvent(group_id)
        event_.set_user_id(user_qq)
        msg_txt = event_.execute(eat_num)

    res_msg = msg_at + Message(msg_txt)
    await bot.send(event=event, message=res_msg)


@bread_rob.handle()
async def _(bot: Bot, event: Event, args: Message = CommandArg(), cmd: Message = RawCommand()):
    try:
        user_qq, group_id, name, msg_at, thing = await pre_get_data(event, bot, cmd, cmd_rob_ori)
    except CommandError:
        return

    robbed_qq = None
    rob_num = None
    for arg in args:
        if arg.type == "at":
            robbed_qq = arg.data.get("qq", "")
        if arg.type == "text":
            text = arg.data.get("text")
            try:
                rob_num = get_num_arg(text, RobEvent, group_id)
            except ArgsError as e:
                await bot.send(event=event, message=str(e))
                return

    if not robbed_qq:
        if bread_config.is_random_robbed:
            all_data = BreadDataManage(group_id).get_all_data()
            all_qq = [x.user_id for x in all_data if x.bread_num >= bread_config.min_rob and x.user_id != user_qq]
            if not all_qq:
                await bot.send(event=event, message="没有可以抢的人w")
                return
            robbed_qq = random.choice(all_qq)
            try:
                robbed_name = await get_nickname(bot, robbed_qq, group_id)
            except ActionFailed:  # 群员不存在
                robbed_name = await get_nickname(bot, robbed_qq)
        else:
            await bot.send(event=event, message="不支持随机抢！请指定用户进行抢")
            return
    else:
        robbed_name = await get_nickname(bot, robbed_qq, group_id)

    wait_time = cd_wait_time(group_id, user_qq, Action.ROB)
    if wait_time > 0:
        msg_txt = f"您还得等待{wait_time // 60}分钟才能抢{thing}w"
    elif wait_time < 0:
        msg_txt = f"你被禁止抢{thing}啦！{(abs(wait_time) + CD.ROB.value) // 60}分钟后才能抢哦！"
    else:
        event_ = RobEvent(group_id)
        event_.set_user_id(user_qq)
        event_.set_other_id(robbed_qq, robbed_name)
        msg_txt = event_.execute(rob_num)

    res_msg = msg_at + msg_txt
    await bot.send(event=event, message=res_msg)


@bread_give.handle()
async def _(bot: Bot, event: Event, args: Message = CommandArg(), cmd: Message = RawCommand()):
    try:
        user_qq, group_id, name, msg_at, thing = await pre_get_data(event, bot, cmd, cmd_give_ori)
    except CommandError:
        return

    given_qq = None
    give_num = None
    for arg in args:
        if arg.type == "at":
            given_qq = arg.data.get("qq", "")
        if arg.type == "text":
            text = arg.data.get("text")
            try:
                give_num = get_num_arg(text, GiveEvent, group_id)
            except ArgsError as e:
                await bot.send(event=event, message=str(e))
                return

    if not given_qq:
        if bread_config.is_random_given:
            all_data = BreadDataManage(group_id).get_all_data()
            all_qq = [x.user_id for x in all_data if x.bread_num and x.user_id != user_qq]
            if not all_qq:
                await bot.send(event=event, message="没有可以赠送的人w")
                return
            given_qq = random.choice(all_qq)
            try:
                given_name = await get_nickname(bot, given_qq, group_id)
            except ActionFailed:  # 群员不存在
                given_name = await get_nickname(bot, given_qq)
        else:
            await bot.send(event=event, message="不支持随机赠送！请指定用户进行赠送")
            return
    else:
        given_name = await get_nickname(bot, given_qq, group_id)

    wait_time = cd_wait_time(group_id, user_qq, Action.GIVE)
    if wait_time > 0:
        msg_txt = f"您还得等待{wait_time // 60}分钟才能送{thing}w"
    elif wait_time < 0:
        msg_txt = f"你被禁止送{thing}啦！{(abs(wait_time) + CD.GIVE.value) // 60}分钟后才能赠送哦！"
    else:
        event_ = GiveEvent(group_id)
        event_.set_user_id(user_qq)
        event_.set_other_id(given_qq, given_name)
        msg_txt = event_.execute(give_num)

    res_msg = msg_at + msg_txt
    await bot.send(event=event, message=res_msg)


@bread_bet.handle()
async def _(bot: Bot, event: Event, args: Message = CommandArg(), cmd: Message = RawCommand()):
    try:
        user_qq, group_id, name, msg_at, thing = await pre_get_data(event, bot, cmd, cmd_bet_ori)
    except CommandError:
        return

    wait_time = cd_wait_time(group_id, user_qq, Action.BET)
    if wait_time > 0:
        msg_txt = f"您还得等待{wait_time // 60}分钟才能猜拳w"
        await bot.send(event=event, message=msg_at + msg_txt)
        return
    elif wait_time < 0:
        msg_txt = f"你被禁止猜拳啦！{(abs(wait_time) + CD.BET.value) // 60}分钟后才能猜拳哦！"
        await bot.send(event=event, message=msg_at + msg_txt)
        return
    else:
        texts = args.extract_plain_text().split()
        ges = texts[0] if texts else ''
        bet_num = None
        if len(texts) == 2:
            bet_txt = texts[1]
            try:
                bet_num = get_num_arg(bet_txt, BetEvent, group_id)
            except ArgsError as e:
                await bot.send(event=event, message=str(e))
                return

        if ges not in ["石头", "剪刀", "布"]:
            await bot.send(event=event, message=f"没有{ges}这种东西啦！请输入“石头”或“剪刀”或“布”！例如 ’/bet 石头‘ ")
            return
        if ges == "石头":
            ges_ = BetEvent.G(0)
        elif ges == "布":
            ges_ = BetEvent.G(1)
        else:
            ges_ = BetEvent.G(2)

        event_ = BetEvent(group_id)
        event_.set_user_id(user_qq)
        event_.set_user_gestures(ges_)
        msg_txt = event_.execute(bet_num)

        res_msg = msg_at + msg_txt
        await bread_bet.finish(res_msg)


@bread_check.handle()
async def _(event: Event, bot: Bot, args: Message = CommandArg(), cmd: Message = RawCommand()):
    try:
        user_qq, group_id, name, msg_at, thing = await pre_get_data(event, bot, cmd, cmd_check_ori)
    except CommandError:
        return

    checked_qq = user_qq
    for arg in args:
        if arg.type == "at":
            checked_qq = arg.data.get("qq", "")
    if checked_qq == user_qq:
        user_data = BreadDataManage(group_id).get_bread_data(user_qq)
        msg = f"你现在拥有{user_data.bread_num}个{thing}，等级为Lv.{user_data.level}，排名为{user_data.no}！"
    else:
        checked_name = await get_nickname(bot, checked_qq, group_id)
        checked_data = BreadDataManage(group_id).get_bread_data(checked_qq)
        msg = f"{checked_name} 现在拥有{checked_data.bread_num}个{thing}，等级为Lv.{checked_data.level}，排名为{checked_data.no}！"

    await bot.send(event=event, message=msg_at + msg)


@bread_log.handle()
async def _(event: Event, bot: Bot, args: Message = CommandArg(), cmd: Message = RawCommand()):
    try:
        user_qq, group_id, name, msg_at, thing = await pre_get_data(event, bot, cmd, cmd_log_ori)
    except CommandError:
        return

    add_arg = args.extract_plain_text().strip()
    if add_arg:
        action_args = ["买", "吃", "抢", "赠送", "猜拳"]
        if add_arg in action_args:
            val_index = action_args.index(add_arg)
            action = Action(val_index)
            data = BreadDataManage(group_id).get_action_log(action)
            name = await get_nickname(bot, data.user_id, group_id)
            attr_val = BreadDataManage.LOG_COLUMN[val_index].lower()
            app_msg = ["哇好有钱！", "好能吃，大胃王！", "大坏比！", "我超，带好人！", "哇塞，赌狗！"]
            msg = f"{add_arg}次数最多是{name}！共{getattr(data, attr_val)}次！" + app_msg[val_index]
            await bot.send(event=event, message=msg)
            return
        else:
            msg = f'没有{add_arg}这个操作啦！只有"买"，"吃"，"抢"，"赠送"，"猜拳" 例如：/logb 买'
            await bot.send(event=event, message=msg_at + msg)
            return

    checked_qq = user_qq
    for arg in args:
        if arg.type == "at":
            checked_qq = arg.data.get("qq", "")
    if checked_qq == user_qq:
        user_log = BreadDataManage(group_id).get_log_data(user_qq)
        msg = f"你共购买{user_log.buy_times}次，吃{user_log.eat_times}次，抢{user_log.rob_times}次，" \
              f"赠送{user_log.give_times}次，猜拳{user_log.bet_times}次！"
    else:
        checked_name = await get_nickname(bot, checked_qq, group_id)
        checked_log = BreadDataManage(group_id).get_log_data(checked_qq)
        msg = f"{checked_name}共购买{checked_log.buy_times}次，吃{checked_log.eat_times}次，抢{checked_log.rob_times}次，" \
              f"赠送{checked_log.give_times}次，猜拳{checked_log.bet_times}次！"
    await bot.send(event=event, message=msg_at + msg)


@bread_help.handle()
async def _(event: Event, bot: Bot, cmd: Message = RawCommand()):
    try:
        user_qq, group_id, name, msg_at, thing = await pre_get_data(event, bot, cmd, cmd_help_ori)
    except CommandError:
        return

    msg = f"""       🍞商店使用说明🍞
指令	        说明
买{thing}    	购买随机{thing}
啃{thing}	    吃随机{thing}
抢{thing}+@	  抢随机{thing}
送{thing}+@	  送随机{thing}
赌{thing}+""	猜拳赌随机{thing}
{thing}记录+""   查看操作次数最多的人
{thing}记录+@    查看操作次数
查看{thing}+@    查看{thing}数据
{thing}排行+	    本群排行榜top5
更多详情见本项目地址：
https://github.com/Mai-icy/nonebot-plugin-bread-shop"""
    await bot.send(event=event, message=msg)


@bread_top.handle()
async def _(bot: Bot, event: Event, args: Message = CommandArg(), cmd: Message = RawCommand()):
    try:
        user_qq, group_id, name, msg_at, thing = await pre_get_data(event, bot, cmd, cmd_top_ori)
    except CommandError:
        return
    args_list = args.extract_plain_text().strip().split()
    if not all(arg.isdigit() for arg in args_list):
        await bot.send(event=event, message="请输入数字！")
        return

    if len(args_list) == 1:
        # 指定查看排行榜区间 从 1 - end(即args_list[0])
        if int(args_list[0]) > 10 or int(args_list[0]) < 1:
            await bot.send(event=event, message="超出范围了！")
            return
        msg = await get_group_top(bot, group_id, thing, end=int(args_list[0]))
    elif len(args_list) == 2:
        # 指定查看排行榜区间 start - end
        end = int(args_list[1])
        start = int(args_list[0])
        if end - start >= 10 or start > end or start < 1:
            await bot.send(event=event, message="超出范围了！")
            return
        msg = await get_group_top(bot, group_id, thing, start=start, end=end)
    elif len(args_list) == 0:
        msg = await get_group_top(bot, group_id, thing)
    else:
        await bot.send(event=event, message="参数非法！")
        return

    await bot.send(event=event, message=msg)


async def get_group_id(session_id):
    """获取群号"""
    res = re.findall("_(.*)_", session_id)
    group_id = res[0]

    # 调整是否全局分群
    for zone_pair in bread_config.group_database.items():
        if group_id in zone_pair[1]:
            return zone_pair[0]

    if bread_config.global_database:
        return "global"
    else:
        return group_id


async def get_group_top(bot: Bot, group_id, thing, start=1, end=5) -> Message:
    """获取群内（或全局）排行榜"""
    if group_id == "global":
        group_member_list = []
    else:
        group_member_list = await bot.get_group_member_list(group_id=int(group_id))

    user_id_list = {info['user_id'] for info in group_member_list}
    all_data = BreadDataManage(group_id).get_all_data()
    num = 0
    append_text = f"🍞本群{thing}排行top！🍞\n"
    for data in all_data:
        if int(data.user_id) in user_id_list or group_id == "global":
            num += 1
            if start <= num <= end:
                name = await get_nickname(bot, data.user_id, group_id)
                append_text += f"top{num} : {name} Lv.{data.bread_eaten // LEVEL}，拥有{thing}{data.bread_num}个\n"
            if num > end:
                break

    append_text += "大家继续加油w！"
    return Message(append_text)


async def get_nickname(bot: Bot, user_id, group_id=None):
    """获取用户的昵称，若在群中则为群名片，不在群中为qq昵称"""
    if group_id and group_id != "global" and group_id not in bread_config.group_database.keys():
        info = await bot.get_group_member_info(group_id=int(group_id), user_id=int(user_id))
        other_name = info.get("card", "") or info.get("nickname", "")
        if not other_name:
            info = await bot.get_stranger_info(user_id=int(user_id))
            other_name = info.get("nickname", "")
    else:
        info = await bot.get_stranger_info(user_id=int(user_id))
        other_name = info.get("nickname", "")
    return other_name


def get_num_arg(text, event_type, group_id):
    """
    获取指令中的操作数量
    例： ”/give @用户 10“ 中的 10
    """
    text = text.strip()
    if text:
        if event_type(group_id).is_random():
            raise ArgsError("本群不可指定其它参数！请正确使用'@'")
        elif not text.isdigit():
            raise ArgsError("请输入数字！")
        else:
            return int(text)
    else:
        return None


async def pre_get_data(event, bot, cmd, cmd_ori):
    user_qq = event.get_user_id()
    group_id = await get_group_id(event.get_session_id())
    name = await get_nickname(bot, user_qq, group_id)

    if bread_config.is_at_valid:
        msg_at = Message(f"[CQ:at,qq={user_qq}]")  # at生效
    else:
        msg_at = Message("@" + name)  # at不生效，为纯文本

    things_ = bread_config.special_thing_group.get(group_id, bread_config.bread_thing)

    if isinstance(things_, list):
        if all((not cmd[1:] in cmd_ori and thing not in cmd) for thing in things_):
            # 指令物品不匹配
            raise CommandError
        thing = things_[0]
    else:
        if not cmd[1:] in cmd_ori and things_ not in cmd:
            raise CommandError
        thing = things_

    if (bread_config.global_bread and group_id in bread_config.black_bread_groups) or \
            (not bread_config.global_bread and group_id not in bread_config.white_bread_groups):
        await bot.send(event=event, message=f"本群已禁止{thing}店！请联系bot管理员！")
        raise CommandError

    return user_qq, group_id, name, msg_at, thing


class ArgsError(ValueError):
    pass


class CommandError(ValueError):
    pass


@driver.on_shutdown
async def close_db():
    BreadDataManage.close_dbs()



#!/usr/bin/python
# -*- coding:utf-8 -*-
import random
from functools import wraps
from datetime import datetime

from .bread_handle import Action
from .bread_operate import RobEvent, EatEvent, BuyEvent, GiveEvent, BetEvent, _Event
from .config import MIN, MAX, LEVEL, bread_config

rob_events = []
eat_events = []
buy_events = []
give_events = []
bet_events = []


# 特殊事件修饰器
def probability(value, action: Action, *, priority: int = 5, group_id_list: list = None):
    def wrapper(func):
        @wraps(func)
        def inner(*args, **kwargs):
            if random.random() < value:
                return func(*args, **kwargs)
            else:
                return None
        setattr(inner, "priority", priority)
        setattr(inner, "group_id_list", group_id_list)
        events_lists = (buy_events, eat_events, rob_events, give_events, bet_events)
        if action == Action.ALL:
            for events in events_lists:
                events.append(inner)
        else:
            event_list = events_lists[action.value]
            event_list.append(inner)
        return inner
    return wrapper


# region 购买特殊事件
@probability(0.01, Action.BUY, priority=5)
def buy_event_gold_bread(event: BuyEvent):
    buy_num = event.action_num + 20
    new_bread_num = event.bread_db.add_bread(event.user_id, buy_num)
    new_bread_no = event.bread_db.update_no(event.user_id)
    append_text = f"出现了！黄金{event.thing}！计为{buy_num}个！现在一共拥有{new_bread_num}个{event.thing}！" \
                  f"您的{event.thing}排名为:{new_bread_no}"
    event.bread_db.cd_update_stamp(event.user_id, Action.BUY)
    return append_text


@probability(0.2, Action.BUY, priority=5)
def buy_event_poverty_relief(event: BuyEvent):
    if event.user_data.bread_num > 10:
        return
    buy_num = event.action_num + 10
    new_bread_num = event.bread_db.add_bread(event.user_id, buy_num)
    new_bread_no = event.bread_db.update_no(event.user_id)
    append_text = f"{event.thing}店看你{event.thing}太少了，送了你{buy_num}个！现在一共拥有{new_bread_num}个{event.thing}！" \
                  f"您的{event.thing}排名为:{new_bread_no}"
    event.bread_db.cd_update_stamp(event.user_id, Action.BUY)
    return append_text


@probability(0.8, Action.BUY, priority=4)
def buy_event_too_much(event: BuyEvent):
    if event.user_data.bread_num < 90:
        return
    append_text = f"你{event.thing}太多啦！不卖给你了，快吃！现在一共拥有{event.user_data.bread_num}个{event.thing}！" \
                  f"您的{event.thing}排名为:{event.user_data.no}"
    event.bread_db.cd_update_stamp(event.user_id, Action.BUY)
    return append_text


@probability(0.025, Action.BUY, priority=5)
def buy_event_find_bad(event: BuyEvent):
    if event.user_data.bread_num < MAX.BUY.value:
        return
    bad_num = event.action_num
    append_text = f"你刚要去买{event.thing}，发现自己{bad_num}个{event.thing}坏掉了！好难过，不想买啦！"
    event.bread_db.reduce_bread(event.user_id, bad_num)
    event.bread_db.cd_update_stamp(event.user_id, Action.BUY)
    return append_text
# endregion


# region 吃特殊事件
@probability(0.2, Action.EAT, priority=5)
def eat_event_too_much_bread(event: EatEvent):
    if event.user_data.bread_num < 90:
        return
    txt = event.normal_event()
    txt += f"\n你{event.thing}太多啦，我允许你再吃一次吧！"
    event.bread_db.cd_refresh(event.user_id, Action.EAT)
    return txt


@probability(0.2, Action.EAT, priority=5)
def eat_event_not_enough(event: EatEvent):
    eat_num = event.action_num
    event.bread_db.reduce_bread(event.user_id, eat_num)
    event.bread_db.add_bread(event.user_id, eat_num, Action.EAT)
    append_text = f"成功吃掉了{eat_num}个{event.thing}w！还是好饿，还可以继续吃！"
    event.bread_db.update_no(event.user_id)
    return append_text


@probability(0.1, Action.EAT, priority=5)
def eat_event_much(event: EatEvent):
    if event.user_data.bread_num <= MAX.EAT.value * 2:
        return
    eat_num = event.action_num * 2
    event.bread_db.reduce_bread(event.user_id, eat_num)
    event.bread_db.add_bread(event.user_id, eat_num, Action.EAT)
    append_text = f"成功吃掉了{eat_num}个{event.thing}w！吃太多啦，撑死了，下次吃多等30分钟！"
    event.bread_db.update_no(event.user_id)
    event.bread_db.cd_ban_action(event.user_id, Action.EAT, 1800)
    return append_text


@probability(0.07, Action.EAT, priority=5)
def eat_event_steal(event: EatEvent):
    eat_num = event.action_num
    event.bread_db.reduce_bread(event.user_id, eat_num)
    append_text = f"你吃{event.thing}被我发现了！我好饿，我帮你吃吧！吃了你{eat_num}个{event.thing}！"
    event.bread_db.update_no(event.user_id)
    return append_text


@probability(0.02, Action.EAT, priority=5)
def eat_event_refresh(event: EatEvent):
    eat_num = event.action_num
    event.bread_db.reduce_bread(event.user_id, eat_num)
    event.bread_db.add_bread(event.user_id, eat_num, Action.EAT)
    append_text = f"成功吃掉了{eat_num}个{event.thing}w！有个{event.thing}超好吃！让你心情舒爽！所有操作刷新！"
    event.bread_db.update_no(event.user_id)
    event.bread_db.cd_refresh(event.user_id, Action.ALL)

    return append_text


@probability(0.01, Action.EAT, priority=5)
def eat_event_good(event: EatEvent):
    eat_num = event.action_num
    event.bread_db.reduce_bread(event.user_id, eat_num)
    event.bread_db.add_bread(event.user_id, eat_num, Action.EAT)
    append_text = f"成功吃掉了{eat_num}个{event.thing}w！吃到了超级{event.thing}！等级加1。"
    event.bread_db.update_no(event.user_id)
    event.bread_db.add_bread(event.user_id, LEVEL, Action.EAT)
    return append_text


@probability(0.015, Action.EAT, priority=5)
def eat_event_bad(event: EatEvent):
    if event.user_data.bread_eaten <= 10:
        return
    append_text = f"{event.thing}坏啦！吃了一个坏{event.thing}！难受死了，等级减1。"
    event.bread_db.reduce_bread(event.user_id, 1)
    event.bread_db.reduce_bread(event.user_id, LEVEL, Action.EAT)
    event.bread_db.update_no(event.user_id)
    return append_text


@probability(0.03, Action.EAT, priority=5)
def eat_event_rob(event: EatEvent):
    eat_num = event.action_num
    event.bread_db.reduce_bread(event.user_id, eat_num)
    event.bread_db.add_bread(event.user_id, eat_num, Action.EAT)
    event.bread_db.update_no(event.user_id)

    append_text = f"成功吃掉了{eat_num}个{event.thing}w！让你充满了力量！刷新抢的冷却！"
    event.bread_db.cd_refresh(event.user_id, Action.ROB)
    event.bread_db.cd_update_stamp(event.user_id, Action.EAT)
    return append_text


@probability(0.025, Action.EAT, priority=5)
def eat_event_find_bad(event: EatEvent):
    if event.user_data.bread_num < MAX.EAT.value:
        return
    bad_num = event.action_num
    append_text = f"你刚想去吃{event.thing}，发现自己{bad_num}个{event.thing}坏掉了！好难过，不想吃啦！"
    event.bread_db.reduce_bread(event.user_id, bad_num)
    event.bread_db.cd_update_stamp(event.user_id, Action.EAT)
    return append_text
# endregion


# region 抢特殊事件
@probability(0.09, Action.ROB, priority=5)
def rob_event_fail(event: RobEvent):
    loss_num = random.randint(0, min(MAX.ROB.value, event.user_data.bread_num))
    new_bread_num = event.bread_db.reduce_bread(event.user_id, loss_num)
    event.bread_db.update_no(event.user_id)
    append_text = f"抢{event.thing}失败啦！被{event.other_name}反击，你丢失{loss_num}个{event.thing}！" \
                  f"你现在拥有{new_bread_num}个{event.thing}！"
    event.bread_db.cd_update_stamp(event.user_id, Action.ROB)
    return append_text


@probability(0.07, Action.ROB, priority=5)
def rob_event_police(event: RobEvent):
    append_text = f"你抢{event.thing}被警察抓住了！你真的太坏了！下次抢{event.thing}时间多等40min！"
    event.bread_db.cd_ban_action(event.user_id, Action.ROB, 2400)
    return append_text


@probability(0.1, Action.ROB, priority=5)
def rob_event_addiction(event: RobEvent):
    append_text = event.normal_event()
    append_text += "有点上瘾，你想再抢一次！"
    event.bread_db.cd_refresh(event.user_id, Action.ROB)
    return append_text


@probability(0.05, Action.ROB, priority=5)
def rob_event_police2(event: RobEvent):
    if event.user_data.bread_num < event.action_num:
        return
    loss_num = event.action_num
    append_text = f"你抢{event.thing}被我抓住了！你真的太坏了！我要罚你{loss_num}个{event.thing}！"
    event.bread_db.reduce_bread(event.user_id, loss_num)
    event.bread_db.update_no(event.user_id)
    event.bread_db.cd_update_stamp(event.user_id, Action.ROB)
    return append_text


@probability(0.05, Action.ROB, priority=5)
def rob_event_hungry(event: RobEvent):
    append_text = f"太饿了！什么都没抢到，但是你想吃东西！吃{event.thing}冷却刷新！"
    event.bread_db.cd_refresh(event.user_id, Action.EAT)
    event.bread_db.cd_update_stamp(event.user_id, Action.ROB)
    return append_text


@probability(0.05, Action.ROB, priority=5)
def rob_event_much(event: RobEvent):
    if event.other_data.bread_num < event.action_num * 2:
        return
    rob_num = event.action_num * 2
    append_text = f"抢劫大成功！抢了{event.other_name}{rob_num}个{event.thing}！"

    event.bread_db.add_bread(event.user_id, rob_num)
    event.bread_db.reduce_bread(event.other_id, rob_num)
    event.bread_db.update_no(event.user_id)
    event.bread_db.update_no(event.other_id)

    event.bread_db.cd_update_stamp(event.user_id, Action.ROB)
    return append_text
# endregion


# region 送特殊事件
@probability(0.1, Action.GIVE, priority=5)
def give_event_commission(event: GiveEvent):
    if event.user_data.bread_num <= MAX.GIVE.value * 2:
        return
    give_num = event.action_num
    user_num = event.bread_db.reduce_bread(event.user_id, give_num * 2)
    event.bread_db.add_bread(event.other_id, give_num)
    append_text = f"哇！这么多{event.thing}，你送了{give_num}个给{event.other_name}！" \
                  f"再给我{give_num}个吧嘿嘿！你现在有{user_num}个{event.thing}！"
    event.bread_db.update_no(event.user_id)
    event.bread_db.update_no(event.other_id)
    event.bread_db.cd_update_stamp(event.user_id, Action.GIVE)
    return append_text


@probability(0.1, Action.GIVE, priority=5)
def give_event_lossless(event: GiveEvent):
    give_num = event.action_num
    event.bread_db.add_bread(event.other_id, give_num)
    append_text = f"看在你如此善良的份上，我帮你送吧！送了{give_num}给{event.other_name}，你不损失{event.thing}！"
    event.bread_db.update_no(event.other_id)
    event.bread_db.cd_update_stamp(event.user_id, Action.GIVE)
    return append_text


@probability(0.06, Action.GIVE, priority=5)
def give_event_lossless2(event: GiveEvent):
    give_num = event.action_num
    give_num2 = random.randint(MIN.GIVE.value, min(MAX.GIVE.value, event.user_data.bread_num))
    event.bread_db.add_bread(event.other_id, give_num)
    event.bread_db.add_bread(event.user_id, give_num2)
    append_text = f"大好人！我帮你送吧！送了{give_num}给{event.other_name}，我再送你{give_num2}个！"
    event.bread_db.update_no(event.other_id)
    event.bread_db.cd_update_stamp(event.user_id, Action.GIVE)
    return append_text
# endregion


# region 猜拳特殊事件
@probability(0.1, Action.BET, priority=5)
def bet_event_addiction(event: BetEvent):
    append_text = event.normal_event()
    append_text += "有点上瘾，你想再来一把！"
    event.bread_db.cd_refresh(event.user_id, Action.BET)
    return append_text


@probability(0.07, Action.BET, priority=5)
def bet_event_police(event: BetEvent):
    append_text = f"你赌{event.thing}被警察抓住了！你不赌，我不赌，和谐幸福跟我走！下次赌{event.thing}时间多等40min！"
    event.bread_db.cd_ban_action(event.user_id, Action.BET, 2400)
    return append_text


@probability(0.04, Action.BET, priority=5)
def bet_event_cheat(event: BetEvent):
    loss_num = event.action_num
    append_text = f"我出三只手！石头剪刀布！你输啦！给我{loss_num}个{event.thing}！"
    event.bread_db.reduce_bread(event.user_id, loss_num)
    event.bread_db.update_no(event.user_id)
    event.bread_db.cd_update_stamp(event.user_id, Action.BET)
    return append_text


@probability(0.04, Action.BET, priority=5)
def bet_event_much(event: BetEvent):
    if event.user_data.bread_num < event.action_num * 2:
        return
    event.action_num = event.action_num * 2
    append_text = event.normal_event()
    append_text = "加大面包量！" + append_text
    return append_text
# endregion



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['nonebot_plugin_bread_shop']

package_data = \
{'': ['*']}

install_requires = \
['nonebot-adapter-onebot>=2.0.0-beta.1,<3.0.0', 'nonebot2>=2.0.0-beta.2,<3.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-bread-shop',
    'version': '0.1.14',
    'description': 'nobebot2面包店插件',
    'long_description': '# nonebot-plugin-bread-shop\n\n![OSCS Status](https://www.oscs1024.com/platform/badge/Mai-icy/nonebot-plugin-bread-shop.svg?size=small)\n[![VERSION](https://img.shields.io/pypi/v/nonebot-plugin-bread-shop)](https://pypi.org/project/nonebot-plugin-bread-shop/)\n![GitHub license](https://img.shields.io/badge/python-3.7+-blue.svg)\n[![GitHub license](https://img.shields.io/github/license/Mai-icy/nonebot-plugin-bread-shop)](https://github.com/Mai-icy/nonebot-plugin-bread-shop/blob/main/LICENSE)\n![Lines of code](https://img.shields.io/tokei/lines/github/Mai-icy/nonebot-plugin-bread-shop)\n\n\n---\n\n\n## ⚠️警告须知！\n\n本插件不适宜用于群人员较多的群，经过测试，本功能具有极大上瘾性，容易造成bot刷屏，影响正常聊天！\n\n## 📄介绍\n\n面包小游戏，用户可以通过“买”，“吃”，“抢”，“送”，”猜拳“操作来获取面包和使用面包。\n\n将会记录所有用户的面包数据进行排行\n\n所有的操作都可能产生特殊面包事件哦！\n\n一起来买面包吧！\n\n> 注：**面包数据库一个群一个，排行均属于群内排行，不同群所有数据不相干。**\n\n## 💿安装\n- 使用 nb-cli\n\n```shell\nnb plugin install nonebot-plugin-bread-shop\n```\n\n- 使用 pip\n\n```shell\npip install nonebot-plugin-bread-shop\n```\n\n## 🤔使用\n\n| 指令 | 说明 | 其它形式 |\n|:-----:|:----:|:----:|\n| 买面包 | 购买随机数量面包 |buy，🍞|\n| 啃面包 | 吃随机数量面包 |eat，🍞🍞，吃面包|\n| 抢面包 + @指定用户 | 抢指定用户随机数量面包 |rob，🍞🍞🍞|\n| 送面包 + @指定用户 | 送指定用户随机数量面包 |give，送|\n| 查看面包 + @指定用户 | 查看指定用户的面包数据 |check，偷看面包，查看面包|\n| 面包记录 + @指定用户 | 查看指定用户的操作次数 |logb，记录|\n| 面包记录 + “买吃抢赠送猜拳” | 查看操作次数最多的人 |logb，记录|\n| 赌面包 + “石头剪刀布” | 和bot进行猜拳赌随机数量面包 |bet，面包猜拳|\n| 面包帮助 | 送指定用户随机数量面包 |breadhelp，helpb|\n| 面包排行 | 发送面包店操作指南（可加查询范围） |breadtop，面包排名|\n\n## ⚙️自定义配置\n\n**配置方式**：请在 `NoneBot` 全局配置文件中添加以下配置项即可。\n\n**参数说明**：\n\n**bread_thing**：可修改默认的“面包”改为其他物品例如： “炸鸡”，“蛋糕”等等（全局）\n\n**special_thing_group** ：分群设置物品例如：{"群号": "炸鸡"}\n\n> 可设置主次词，主次物品词bot将以主物品词进行回复，次物品词只用于触发指令，主关键词为列表第一个元素，次关键词可多个 示例：{"群号": ["炸鸡", "面包", "蛋糕"]}\n\n**global_bread**：面包全局开关\n\n**black_bread_groups**：黑名单\n\n**white_bread_groups**：白名单\n\n**level_bread_num**：修改升级一级所需要吃的面包数量，默认为10\n\n(注意：等级不被数据库记录数据库只记录已经吃了的数量，修改该值会使用户等级变化，变化可逆)\n\n**cd_buy,cd_eat,cd_rob,cd_give,cd_bet**：操作冷却（单位：秒）\n\n**max_buy,max_eat,max_rob,max_give,max_bet**：操作随机值上限\n\n**min_buy,min_eat,min_rob,min_give,min_bet**：操作随机值下限\n\n**is_random_give,is_random_eat等**：设置是否操作值都由随机值决定(全局)\n\n**is_random_robbed**： 抢面包操作不指定群员可随机抢\n\n**is_random_given**： 送面包操作不指定群员可随机送\n\n(注意：改为False之后用户可以通过 "操作名 + @ + 数量" 或 "操作名 + 数量" 达到效果)\n\n**special\\_操作名_group**：设置特别处理的群 （示例： {"群号": bool}）\n\n\n**global_database**: 数据库是否全局，若设置了group_database，以其为优先，全局数据库文件夹名为"global"\n\n**group_database**: 合并一些群的数据库 分组id将作为文件夹名 例：{"分组id":["群号1", "群号2", "群号3"]}\n\n**is_at_valid**: 选择是否启用有效at （由于担心风控影响，可自行选择）\n\n>注意：此处的分组id将生效于 special_thing_group 的设置 示例{"分组id": "炸鸡"}，原来的设置将失效。特殊事件同理设置的群聊id同理请改为组id\n\n\n详情请见config.py文件\n\n## 🍞自定义事件\n\n在**bread_event.py**中可以编写特殊事件！\n\n**可以在[issue](https://github.com/Mai-icy/nonebot-plugin-bread-shop/issues/5)中提供你的点子！**\n\n特殊事件模板：\n\ngroup_id_list默认为全部群聊\n\npriority默认为5，数字越低越优先，优先级相同的事件先后顺序每次随机\n\n```python\n@probability(概率, Action.操作, priority=优先级, group_id_list=["群号1", "群号2"])\ndef 函数名(event: 操作):\n    # event.user_data 可以查看操作的用户的面包数据\n    # event.user_id   可以获取操作的用户的id（qq）\n    # event.user_id   可以获取操作的用户的id（qq）\n    # event.bread_db.reduce_bread(event.user_id, eat_num) 减少用户面包数量\n    # event.bread_db.reduce_bread(event.user_id, eat_num) 增加用户面包数量\n    # event.bread_db.add_bread(event.user_id, eat_num, "BREAD_EATEN")  增加用户面包食用量\n    # event.bread_db.update_no(event.user_id)  更新用户排名\n    # event.bread_db.ban_user_action(event.user_id, Action.EAT, 1800) 禁止用户操作\n    # event.bread_db.cd_refresh(event.user_id, Action.EAT)        刷新用户CD\n    # event.bread_db.update_cd_stamp(event.user_id, Action.GIVE)  重置用户CD\n    # 等等见源码\n    return append_text  # 返回回答，由bot发送\n```\n\n特殊事件示例：\n\n```python\n@probability(0.1, Action.EAT, priority=5)\ndef eat_event_much(event: EatEvent):\n    if event.user_data.bread_num <= MAX.EAT.value:\n        return\n    eat_num = random.randint(MAX.EAT.value, min(MAX.EAT.value * 2, event.user_data.bread_num))\n    event.bread_db.reduce_bread(event.user_id, eat_num)\n    event.bread_db.add_bread(event.user_id, eat_num, "BREAD_EATEN")\n    append_text = f"成功吃掉了{eat_num}个面包w！吃太多啦，撑死了，下次吃多等30分钟！"\n    event.bread_db.update_no(event.user_id)\n    event.bread_db.ban_user_action(event.user_id, Action.EAT, 1800)\n    return append_text\n```\n\n若想要设置买面包打烊时间如：\n\n```python\n@probability(1, Action.EAT, priority=1, group_id_list=["群号1", "群号2"])\ndef closing_time(event: EatEvent):\n    if 判断时间:\n        event.bread_db.reduce_user_log(event.user_id, Action.EAT)  # 防止记录\n    \treturn "打烊"\n```\n\n其他注意点：\n\nevent.normal_event()为事件正常进行全流程并返回原来的话。\n\n例如：\n\n```python\n@probability(0.1, Action.BET, priority=5)\ndef bet_event_addiction(event: BetEvent):\n    append_text = event.normal_event()\n    append_text += " 有点上瘾，你想再来一把！"\n    event.bread_db.cd_refresh(event.user_id, Action.BET)\n    return append_text\n```\n\n\n\nreturn None 相当于事件不触发，返回任何字符串都认定为事件触发\n\n',
    'author': 'Mai Icy',
    'author_email': 'maiicy@Foxmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Mai-icy/nonebot-plugin-bread-shop',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.7,<4.0.0',
}


setup(**setup_kwargs)
